package util.concurrent;

import java.util.concurrent.atomic.AtomicLong;
import java.util.concurrent.atomic.LongAccumulator;
import java.util.concurrent.atomic.LongAdder;

/**
 * Thread-safe counter.
 *
 * Overflow behavior:
 * Overflows from {@link Long#MAX_VALUE} to <code>zero</code>.
 * 
 * Example use-case:
 * The counter is for when you actually want to use each value.  If you just want
 * to do a summation from different threads, ie you are not interested in using
 * each value in a threadsafe way, but just interested in the final sum - then
 * you might get better performance using Java 8's {@link LongAdder} or
 * {@link LongAccumulator}.
 *
 * @see "https://www.baeldung.com/java-longadder-and-longaccumulator"
 * @see "http://blog.palominolabs.com/2014/02/10/java-8-performance-improvements-longadder-vs-atomiclong/"
 */
public class Counter {
    
    private final AtomicLong atomicLong;
    private final long initialValue;
	
    public Counter() {
    	this(0);
    }
    
    public Counter(long initialValue) {
    	this.initialValue = initialValue;
    	this.atomicLong = new AtomicLong(initialValue);
    }
	
    public long incrementAndGet() {
    	if (atomicLong.get() == Long.MAX_VALUE) {
    		synchronized(atomicLong) {
    			if (atomicLong.get() == Long.MAX_VALUE) {
					atomicLong.set(0);
					return 0;
				}
			}
		}
    	return atomicLong.incrementAndGet();
    }
	
    public long get() {
    	return atomicLong.get();
    }
    public void set(long newValue) {
    	atomicLong.set(newValue);
	}
}
