package util.testing;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.concurrent.*;

public abstract class AbstractTest {
	
	/**
	 * 
	 * Example client code:
	 * 
	 * <code>
		List<Callable<Void>> callableList = new LinkedList<>();
		for (int i=0 ; i<5 ; i++) {
			callableList.add(new Callable<Void>() {
				@Override
				public Void call() throws Exception {
					return null;
				}
			});
		}
	 	
	 	testConcurrent(callableList);
	 * </code>
	 */
	protected static <T> void testConcurrent(List<? extends Callable<T>> callableList) throws Exception {
		ExecutorService threadPool = null;
		try {
			// XXX Creating a threadpool synchronously ==> expensive operation
			threadPool = Executors.newWorkStealingPool();
			
			List<Future<T>> futureList = threadPool.invokeAll(callableList);
			
			// meh, disallow further additions to the threadpool
			threadPool.shutdown();
			
			// wait for everything to be uploaded
			if (!threadPool.awaitTermination(15, TimeUnit.MINUTES)) {
				throw new Exception("Asynchronous operations took too long and was terminated...");
			}
			
			for (Future<?> currFuture : futureList) {
				currFuture.get();
			}
		} catch (Exception e) {
			// Unexpected error - aggressively shut down the thread pool
			if (threadPool != null) {
				threadPool.shutdownNow();
			}
			
			throw e;
		}
	}
	
	/*
	 * Convenience methods.
	 */
	protected static void trace(CharSequence logMessage) { // needs to be static so that I can call it from static code segments
		Logger logger = LoggerFactory.getLogger(THIS_CLASS());
		if (logger.isTraceEnabled()) {
			logger.trace(logMessage != null ? logMessage.toString() : "NULL");
		}
	}
	protected static void debug(CharSequence logMessage) { // needs to be static so that I can call it from static code segments
		Logger logger = LoggerFactory.getLogger(THIS_CLASS());
		if (logger.isDebugEnabled()) {
			logger.debug(logMessage != null ? logMessage.toString() : "NULL");
		}
	}
	protected static void info(CharSequence logMessage) { // needs to be static so that I can call it from static code segments
		Logger logger = LoggerFactory.getLogger(THIS_CLASS());
		if (logger.isInfoEnabled()) {
			logger.info(logMessage != null ? logMessage.toString() : "NULL");
		}
	}
	protected static void warn(Throwable e) { // needs to be static so that I can call it from static code segments
		Logger logger = LoggerFactory.getLogger(THIS_CLASS());
		if (logger.isWarnEnabled())
			logger.warn(null, e);
	}
	protected static void warn(CharSequence logMessage) { // needs to be static so that I can call it from static code segments
		Logger logger = LoggerFactory.getLogger(THIS_CLASS());
		if (logger.isWarnEnabled()) {
			logger.warn(logMessage != null ? logMessage.toString() : "NULL");
		}
	}
	protected static void warn(Throwable e, CharSequence logMessage) { // needs to be static so that I can call it from static code segments
		Logger logger = LoggerFactory.getLogger(THIS_CLASS());
		if (logger.isWarnEnabled()) {
			logger.warn((logMessage != null ? logMessage.toString() : "NULL"), e);
		}
	}
	protected static void error(Throwable e) { // needs to be static so that I can call it from static code segments
		Logger logger = LoggerFactory.getLogger(THIS_CLASS());
		if (logger.isErrorEnabled()) {
			logger.error(null, e);
		}
	}
	protected static void error(CharSequence logMessage) { // needs to be static so that I can call it from static code segments
		Logger logger = LoggerFactory.getLogger(THIS_CLASS());
		if (logger.isErrorEnabled()) {
			logger.error(logMessage != null ? logMessage.toString() : "NULL");
		}
	}
	protected static void error(Throwable e, CharSequence logMessage) { // needs to be static so that I can call it from static code segments
		Logger logger = LoggerFactory.getLogger(THIS_CLASS());
		if (logger.isErrorEnabled()) {
			logger.error((logMessage != null ? logMessage.toString() : "NULL"), e);
		}
	}
	
	private static Class<?> THIS_CLASS() {
		Class<?> subClazz;
		try {
			// hacky, but its a test so I'll allow it :)
			subClazz = Class.forName(Thread.currentThread().getStackTrace()[3].getClassName());
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		}
		return subClazz;
	}
}
